@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2014 Geoffrey T. Dairiki <dairiki@dairiki.org>
+# Copyright © 2014–2021 Geoffrey T. Dairiki <dairiki@dairiki.org>
 #
 from __future__ import absolute_import
 
@@ -13,6 +13,15 @@ import pytest
 
 from ._compat import NativeIO, text_
 
+from pyramid_log import (
+    _add_request_attr,
+    logging_disabled,
+    _DottedLookup,
+    Formatter,
+    Missing,
+    _WrapDict,
+)
+
 
 @pytest.fixture
 def current_request(request):
@@ -30,7 +39,6 @@ class TestIntegration(object):
     def logstream(self, monkeypatch):
         """ Patch the root logger to log to an in-memory stream.
         """
-        import pyramid_log
         logstream = NativeIO()
         root = logging.getLogger()
         monkeypatch.setattr(root, 'handlers', [])
@@ -38,8 +46,7 @@ class TestIntegration(object):
         fmt = ("%(asctime)s "
                "%(request.method|-)s %(request.path_qs|-)s : "
                "%(message)s")
-        formatter = pyramid_log.Formatter(fmt, datefmt="<DATE>")
-        handler.setFormatter(formatter)
+        handler.setFormatter(Formatter(fmt, datefmt="<DATE>"))
         root.addHandler(handler)
         return logstream
 
@@ -62,47 +69,27 @@ def log_record():
 
 
 class TestFormatter(object):
-    def make_one(self, *args, **kwargs):
-        from pyramid_log import Formatter
-        return Formatter(*args, **kwargs)
-
     def test_with_explicit_request(self, log_record):
         log_record.request = Request.blank('/', POST={})
-        formatter = self.make_one('%(request.method)s')
+        formatter = Formatter('%(request.method)s')
         assert formatter.format(log_record) == 'POST'
-        assert hasattr(log_record, 'request')
 
     def test_with_threadlocal_request(self, current_request, log_record):
-        formatter = self.make_one('%(request.method)s')
+        formatter = Formatter('%(request.method)s')
         assert formatter.format(log_record) == 'GET'
-        assert not hasattr(log_record, 'request')
 
     def test_with_no_request(self, log_record):
-        formatter = self.make_one('%(request|no request)s')
+        formatter = Formatter('%(request|no request)s')
         assert formatter.format(log_record) == 'no request'
-        assert not hasattr(log_record, 'request')
 
     def test_format_asctime(self, log_record):
-        formatter = self.make_one('asctime=%(asctime)s', datefmt="DATEFMT")
+        formatter = Formatter('asctime=%(asctime)s', datefmt="DATEFMT")
         assert formatter.format(log_record) == 'asctime=DATEFMT'
 
     def test_format_parentheses_in_fallback(self, log_record):
-        formatter = self.make_one('%(request.method|(no request))s')
+        formatter = Formatter('%(request.method|(no request))s')
         assert formatter.format(log_record) == '(no request)'
 
-    def test_format_called_with_log_disabled(self, log_record):
-        manager = logging.root.manager
-
-        class MockRequest(object):
-            @property
-            def disable(self):
-                return manager.disable
-        log_record.request = MockRequest()
-        formatter = self.make_one('%(request.disable)s')
-        assert formatter.format(log_record) == '%d' % log_record.levelno
-        # Check that manager.disable is restored
-        assert not manager.disable
-
 
 class MockObject(object):
     def __init__(self, **kw):
@@ -114,14 +101,53 @@ class MockWrapper(object):
         self.wrapped = d
 
 
-class TestWrapDict(object):
-    def make_one(self, obj, dictwrapper):
-        from pyramid_log import _WrapDict
-        return _WrapDict(obj, dictwrapper)
+class Test_add_request_attr(object):
+    @pytest.fixture
+    def record(self):
+        return MockObject()
 
+    def test_adds_threadlocal_request(self, record, current_request):
+        with _add_request_attr(record):
+            assert record.request is current_request
+        assert not hasattr(record, 'request')
+
+    def test_explicit_request(self, record, current_request):
+        explicit_request = object()
+        record.request = explicit_request
+        with _add_request_attr(record):
+            assert record.request is explicit_request
+        assert record.request is explicit_request
+
+    def test_no_request(self, record):
+        with _add_request_attr(record):
+            assert not hasattr(record, 'request')
+
+    def test_target_is_record(self, record, current_request):
+        with _add_request_attr(record) as record_with_request:
+            assert record is record_with_request
+
+
+class Test_logging_disabled(object):
+    def test(self):
+        assert logging.root.manager.disable == 0
+        with logging_disabled(logging.WARNING):
+            assert logging.root.manager.disable == logging.WARNING
+        assert logging.root.manager.disable == 0
+
+    def test_disables_logging(self, caplog):
+        with logging_disabled(logging.WARNING):
+            logging.warning("a warning")
+            assert len(caplog.records) == 0
+            logging.error("an error")
+            assert len(caplog.records) == 1
+        logging.warning("another warning")
+        assert len(caplog.records) == 2
+
+
+class TestWrapDict(object):
     def test_dict(self):
         obj = MockObject()
-        proxy = self.make_one(obj, MockWrapper)
+        proxy = _WrapDict(obj, MockWrapper)
         assert type(proxy.__dict__) is MockWrapper
         assert proxy.__dict__.wrapped is obj.__dict__
 
@@ -130,7 +156,7 @@ class TestWrapDict(object):
             def m(self):
                 return 'foo'
             x = 'bar'
-        proxy = self.make_one(Obj(), MockWrapper)
+        proxy = _WrapDict(Obj(), MockWrapper)
         assert proxy.m() == 'foo'
         assert proxy.x == 'bar'
         with pytest.raises(AttributeError):
@@ -138,7 +164,7 @@ class TestWrapDict(object):
 
     def test_setattr(self):
         obj = MockObject(x='orig')
-        proxy = self.make_one(obj, MockWrapper)
+        proxy = _WrapDict(obj, MockWrapper)
         proxy.x = 'changed'
         assert proxy.x == 'changed'
         assert obj.x == 'changed'
@@ -148,7 +174,7 @@ class TestWrapDict(object):
 
     def test_delattr(self):
         obj = MockObject(x='orig')
-        proxy = self.make_one(obj, MockWrapper)
+        proxy = _WrapDict(obj, MockWrapper)
         del proxy.x
         assert obj.__dict__ == {}
 
@@ -157,29 +183,25 @@ EURO_SIGN = text_('\N{EURO SIGN}', 'unicode-escape')
 
 
 class TestMissing(object):
-    def make_one(self, key, fallback=None):
-        from pyramid_log import Missing
-        return Missing(key, fallback)
-
     @pytest.mark.parametrize('fallback, expected', [
         (None, "<?attr.name?>"),
         ('foo', "'foo'"),
         (EURO_SIGN, repr(EURO_SIGN)),
         ])
     def test_repr(self, fallback, expected):
-        missing = self.make_one("attr.name", fallback)
+        missing = Missing("attr.name", fallback)
         assert repr(missing) == expected
 
     def test_repr_unicode_key(self):
         # carefully constructed to work in python 3.2
-        missing = self.make_one(EURO_SIGN)
+        missing = Missing(EURO_SIGN)
         assert repr(missing) in (
             r"<?\u20ac?>",              # py2
             "<?%s?>" % EURO_SIGN,       # py3k
             )
 
     def test_str(self):
-        missing = self.make_one('key', "foo")
+        missing = Missing('key', "foo")
         assert str(missing) == "foo"
 
     @pytest.mark.parametrize('fallback, expected', [
@@ -188,7 +210,7 @@ class TestMissing(object):
         ('foo', 0),
         ])
     def test_int(self, fallback, expected):
-        missing = self.make_one('somekey', fallback)
+        missing = Missing('somekey', fallback)
         assert int(missing) == expected
 
     @pytest.mark.parametrize('fallback, expected', [
@@ -196,15 +218,15 @@ class TestMissing(object):
         ('3.14', 3.14),
         ])
     def test_float(self, fallback, expected):
-        missing = self.make_one('somekey', fallback)
+        missing = Missing('somekey', fallback)
         assert float(missing) == expected
 
     def test_float_default_fallback(self):
-        missing = self.make_one('somekey', None)
+        missing = Missing('somekey', None)
         assert math.isnan(float(missing))
 
     def test_float_not_a_number(self):
-        missing = self.make_one('somekey', 'notanumber')
+        missing = Missing('somekey', 'notanumber')
         assert math.isnan(float(missing))
 
     @pytest.mark.parametrize('fallback, asint, asfloat', [
@@ -212,14 +234,14 @@ class TestMissing(object):
         ('12', '12', '12.00'),
         ])
     def test_format(self, fallback, asint, asfloat):
-        missing = self.make_one('key', fallback)
+        missing = Missing('key', fallback)
         assert '%s' % missing == fallback
         assert '%r' % missing == repr(fallback)
         assert '%d' % missing == asint
         assert '%.2f' % missing == asfloat
 
     def test_format_unicode(self):
-        missing = self.make_one('key', EURO_SIGN)
+        missing = Missing('key', EURO_SIGN)
         # missing.__str__()
         assert '%s' % missing in (
             r'\u20ac',                  # py2
@@ -230,25 +252,21 @@ class TestMissing(object):
 
 
 class TestDottedLookup(object):
-    def make_one(self, dict_):
-        from pyramid_log import _DottedLookup
-        return _DottedLookup(dict_)
-
     def test_dotted_attribute_lookup(self):
-        d = self.make_one({'a': MockObject(b=MockObject(c='d'))})
+        d = _DottedLookup({'a': MockObject(b=MockObject(c='d'))})
         assert d['a.b.c'] == 'd'
 
     def test_getitem_lookup(self):
-        d = self.make_one({'a': {'b': 'c'}})
+        d = _DottedLookup({'a': {'b': 'c'}})
         assert d['a.b'] == 'c'
 
     def test_fallback(self):
-        d = self.make_one({'a': MockObject(b='x')})
+        d = _DottedLookup({'a': MockObject(b='x')})
         assert str(d['c|rats']) == 'rats'
         assert str(d['a.c|rats']) == 'rats'
         assert str(d['a.b.c|rats']) == 'rats'
 
     def test_default_fallback(self):
-        d = self.make_one({'a': MockObject(b='x')})
+        d = _DottedLookup({'a': MockObject(b='x')})
         assert str(d['missing-key']) == '<?missing-key?>'
         assert str(d['a.missing-key']) == '<?a.missing-key?>'
